#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a drawing look 
#      inspired by a Line Sketch script by kward1979uk modified/tested by Paul Sherman 

# (c) Dinasset 2020
#
#   History:
#
#   v1.0: 2020-09-30: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
from gimpfu import *
 
import pygtk
import gtk
from sys import argv, platform
from ctypes import *
 
# define here some constant value, when required
FixWH=3000
inAreas=11
TypeDesc=["0=Sobel", "1=Prewitt","2=Gradient","3=Roberts","4=Differential","5=Laplace"]
# predefined colour names
BLOSSOM=(250,198,204,255)
CORAL=(255,182,173,255)
PAPAYA=(246,139,105,255)
POPPY=(251,175,95,255)
CURRY=(250,207,99,255)
SUNSHINE=(255,239,108,255)
CHARTREUSE=(206,215,92,255)
MINT=(210,239,219,255)
CLOVER=(161,206,94,255)
SAGE=(219,223,195,255)
BLUEBELL=(174,209,233,255)
POOL=(162,208,207,255)
PLUM=(204,193,219,255)
CEMENT=(227,223,214,255)
GRAVEL=(178,177,165,255)
PAPER_BAG=(197,171,137,255)
LUXE_BLUSH=(255,242,236,255)
ANTIQUE_GOLD=(179,142,80,255)
SHIMMER_GOLD=(255,226,147,255)
SHIMMER_SILVER=(200,201,208,255)

############################################################################
# Service routines
#---------------------------------------------------------------------------

  
############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
############################################################################
    
############################################################################
# Service routines for GEGL C2G
#---------------------------------------------------------------------------

def load_library (library_name):
    if platform == "linux" or platform == "linux2":
        library_name = library_name + '.so.0'
    elif platform == "win32":
        from ctypes.util import find_library
        library_name = find_library (library_name + "-0")
    else:
        raise BaseException ("TODO")
    return CDLL (library_name)
#import sys
#print (sys.version)
gimp = load_library ('libgimp-2.0')
#print (gimp)
#gimp = load_library ('libgimp-3.0')

def gegl_graph(image, drawable, gegl_graph_string):
    class GeglBuffer(Structure):
        pass
    drawable_id = drawable.ID
    
    gegl = load_library ('libgegl-0.4')
    sucess = gegl.gegl_init (None, None)
    gimp.gimp_drawable_get_shadow_buffer.restype = POINTER (GeglBuffer)
    gimp.gimp_drawable_get_buffer.restype        = POINTER (GeglBuffer)

    x = c_int (); y = c_int (); w = c_int (); h = c_int ()
    non_empty,x,y,w,h = pdb.gimp_drawable_mask_intersect (drawable)
    args = [b"string", c_char_p (gegl_graph_string), c_void_p ()]
    
    if non_empty:
        source = gimp.gimp_drawable_get_buffer (drawable_id)
        target = gimp.gimp_drawable_get_shadow_buffer (drawable_id)
        sucess = gegl.gegl_render_op (source, target, "gegl:gegl", *args)
        gegl.gegl_buffer_flush (target)
        gimp.gimp_drawable_merge_shadow (drawable_id, PushUndo = True)
        gimp.gimp_drawable_update (drawable_id, x, y, w, h)
        gimp.gimp_displays_flush ()

def gegl_c2g(image, drawable):
    radius=300
    samples=4   #4
    iterations=10
    enhance_shadows=True
    gegl_graph_string="gegl:c2g radius=%d samples=%d iterations=%d enhance-shadows=%d" % (radius, samples, iterations, enhance_shadows)
    gegl_graph(image, drawable,gegl_graph_string)
  
#
# ======================================================
# ANCILLARY MODULE : Posterize Drawable thru G'MIC   
# ------------------------------------------------------

def PaletteFromImageB (image,layer,nr_of_colours,iname):
    
    width = layer.width
    height = layer.height
    WH = width+height
    source_name = image.name
    #gimp.message(str(source_name))
    split = source_name.split('(')   # when name comes from image
    source_name = split[0]
    initial = source_name[0:1]
    if initial=="[":    # std name is [name]; name attributed by ofnuts' filter is name without [] 
        end = len(source_name)-2
        source_name = source_name[1:end]    # remove []
    #gimp.message(str(iname))
    # remove previously generated palettes
    Pname="Palette generated by "+source_name
    PnameALL="Palette generated by "
    (nP,lP)=pdb.gimp_palettes_get_list(PnameALL)
    if nP>0:
        x=0
        for x in range(0,nP): 
            pdb.gimp_palette_delete (lP[x])
        #gimp.message("deleted "+str(nP)+" palette(s) generated in previous run(s)")    
        
    PaletteImage = pdb.gimp_image_new (width, height, RGB)                    
    PaletteBG = pdb.gimp_layer_new_from_drawable (layer, PaletteImage)
    PaletteImage.add_layer (PaletteBG, 0)
    pdb.gimp_item_transform_scale (PaletteBG, 0, 0, width, height)
    PaletteBG.mode = LAYER_MODE_NORMAL
    # scale down for performances
    pdb.gimp_image_scale (PaletteImage, int(width*0.5), int(height*0.5))
    
    pdb.plug_in_gmic_qt(PaletteImage, PaletteBG, 1,2 ,
    #[G'MIC] Posterize: fx_posterize 800,30,1,8,0,100,1,0,50,50
    "-v - -fx_posterize 800,20,1,"+str(nr_of_colours)+",0,100,1,0,50,50 ")
    PosterLayer = PaletteImage.layers[0]
    PosterLayer.mode = LAYER_MODE_NORMAL
    PosterLayer.opacity = 100.0  
    PosterLayer.name = "[GMIC] -fx_posterize 150,30,1,"+str(nr_of_colours)+",0,0,0,0,50,50 "
    
    pdb.plug_in_gmic_qt(PaletteImage, PosterLayer, 1,2 ,
    #[G'MIC] Colors to Layers: fx_split_colors 0,256,0,0
    "-v - -fx_split_colors 0,256,0,0 ")
    (num_layers,layer_ids) = pdb.gimp_image_get_layers (PaletteImage)
    generated = num_layers-2    # PaletteBG and PosterLayer
    #gimp.message ("generated "+str(generated)+" layers")
    
    areaNR=0
    area_color=[]   # empty list
    for areaNR in range(0,generated):
        #one_color=pdb.gimp_palette_entry_get_color(temp_palette, areaNR)
        pdb.gimp_context_set_antialias(False)
        pdb.gimp_context_set_feather(False)
        pdb.gimp_context_set_sample_criterion(SELECT_CRITERION_COMPOSITE)
        pdb.gimp_context_set_sample_merged(False)
        pdb.gimp_context_set_sample_threshold(0.0)
        pdb.gimp_context_set_sample_transparent(True)
        # select transparent
        pdb.gimp_image_select_color(PaletteImage, CHANNEL_OP_REPLACE,
                                    PaletteImage.layers[areaNR], (0,0,0,0)) 
        pdb.gimp_selection_invert (PaletteImage)
        (there_is_a_selection, x1, y1, x2, y2) = pdb.gimp_selection_bounds(PaletteImage)
        if there_is_a_selection==True:
            (meanR,B,C,D,E,F) = pdb.gimp_drawable_histogram (PaletteImage.layers[areaNR],
                                                         HISTOGRAM_RED, 0.0, 1.0)
            (meanG,B,C,D,E,F) = pdb.gimp_drawable_histogram (PaletteImage.layers[areaNR],
                                                         HISTOGRAM_GREEN, 0.0, 1.0)
            (meanB,B,C,D,E,F) = pdb.gimp_drawable_histogram (PaletteImage.layers[areaNR],
                                                         HISTOGRAM_BLUE, 0.0, 1.0)
            color = (int(meanR),int(meanG),int(meanB))  # WARNING: int() is necessary !
            area_color.append(color)
            #gimp.message ("color for area "+str(areaNR)+" is "+str(color))
        else:
            gimp.message ("area nr. "+str(areaNR)+" there_is_NO_selection")
            
    pdb.gimp_selection_none(PaletteImage)
    
    generated_palette = pdb.gimp_palette_new (Pname)
    for areaNR in range(0,generated):
        pdb.gimp_palette_add_entry(generated_palette, str(areaNR), area_color[areaNR])
    #gimp.message ("created: "+str(generated_palette))
    
    pdb.gimp_palettes_refresh()
    
    posterized=pdb.gimp_image_flatten (PaletteImage)
    posterized.name = source_name+", posterized with palette creation"
    # scale back for performances
    pdb.gimp_image_scale (PaletteImage, width, height)
    
    return (posterized,generated_palette)
    
############################################################################
#    M   A   I   N
#---------------------------------------------------------------------------

def DrawStyleD (inImage, inDrawable, outimgWH,
                BGpredefcol, BGusercol, outLook,
                doFlatten):
    
    pdb.gimp_context_push()
    
    Name = inDrawable.name
    if '.' in Name:
        split = Name.split(".", 1)
        Name = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
    
    Width = inDrawable.width
    Height = inDrawable.height
    OrigWH = Width+Height
    # image name
    Iname = inImage.name
    split = Iname.split("(", 1)
    Iname = split[0]
    end = len(Iname)-2
    Iname = Iname[1:end]    # remove []
    
    Ratio = FixWH*1.00/OrigWH*1.00
    rWidth = int(Width*Ratio)
    rHeight = int(Height*Ratio)
    
    # Create sketched image
    SketchedImg = pdb.gimp_image_new (Width, Height, RGB_IMAGE)
    isamask = pdb.gimp_item_is_layer_mask (inDrawable)
    if isamask==True:    
        inLayer = pdb.gimp_layer_from_mask (inDrawable)
        SketchedDrw = pdb.gimp_layer_new_from_drawable (inLayer, SketchedImg)
    else:    
        SketchedDrw = pdb.gimp_layer_new_from_drawable (inDrawable, SketchedImg)
    # ofnuts technique to give it a name (unsaved). I set it to the name of the original layer
    #SketchedImg.filename=os.path.join(gimp.user_directory(4),Iname)
    SketchedImg.filename=Iname
    SketchedDrw.name = Name+", to sketch"
    SketchedImg.add_layer (SketchedDrw, 0)
    #
    Original = SketchedDrw.copy()
    SketchedImg.add_layer (Original, 1)
    Original.name = Name+" (original)"
    pdb.gimp_image_scale (SketchedImg, rWidth, rHeight)
    # check whether there is a mask
    usermask = pdb.gimp_layer_get_mask (SketchedDrw)
    #pdb.gimp_message (str(usermask))
    if usermask<>None:
        maskedlayer=True
        OriginalMasked=Original.copy() # this is masked
        SketchedImg.add_layer (OriginalMasked, 2)
        OriginalMasked.name = Name+" (original with mask)"
        pdb.gimp_layer_remove_mask (SketchedDrw, MASK_DISCARD)
        pdb.gimp_image_remove_layer (SketchedImg, Original) # was masked
        Original = SketchedDrw.copy()                       # redo to get it unmasked
        SketchedImg.add_layer (Original, 1)
        Original.name = Name+" (original not masked)"
    else:
        maskedlayer=False
    
    #pdb.gimp_context_pop()
    #pdb.gimp_display_new (SketchedImg)
    #return()
    
    # Equalize SketchedDrw
    pdb.gimp_drawable_equalize (SketchedDrw, False)
    # Posterize SketchedDrw
    inNrAreas=int(inAreas) 
    # make use of the ancillary function "PaletteFromImageB"
    (genPoster, temp_palette) = PaletteFromImageB (SketchedImg,SketchedDrw,inNrAreas,Iname)
    PosterizedDrw = pdb.gimp_layer_new_from_drawable (genPoster, SketchedImg)
    SketchedImg.add_layer (PosterizedDrw, 0)
    PosterizedDrw.name=Name+" Posterized"
    
    #pdb.gimp_context_pop()
    #pdb.gimp_display_new (SketchedImg)
    #return()
    
    EdgeAmount=3.0; EdgeWrap=2; EdgeType=1  #Prewitt
    EdgesDrw = PosterizedDrw.copy()
    SketchedImg.add_layer (EdgesDrw, 0)
    EdgesDrw.name = Name+", sketched (EdgeType"+TypeDesc[EdgeType]+")"
    
    pdb.plug_in_edge (SketchedImg, EdgesDrw, EdgeAmount, EdgeWrap, EdgeType)
      
    #pdb.gimp_display_new (SketchedImg)
    #pdb.gimp_context_pop()
    #return
    
    pdb.gimp_drawable_invert (EdgesDrw, True)
    pdb.gimp_layer_add_alpha (EdgesDrw)
    
    pdb.gimp_drawable_curves_spline (EdgesDrw, HISTOGRAM_VALUE, 6,
                                     ( 0.00,0.15, 0.50,0.35, 1.00,0.60) 
                                    )
    EdgesDrw.mode = LAYER_MODE_LINEAR_BURN  ######MULTIPLY
    EdgesDrw.opacity = 50.0
    
    #pdb.gimp_display_new (SketchedImg)
    #pdb.gimp_context_pop()
    #return
    
    pdb.plug_in_spread (SketchedImg, EdgesDrw, rWidth/30.0, rHeight/30.0) ####
    
    # 2nd Edges (2) start
    # -------------------------------------------------------------------------
    EdgeAmount=2.0; EdgeWrap=2; EdgeType=1  #Prewitt
    
    EdgesDrw2 = Original.copy()
    SketchedImg.add_layer (EdgesDrw2, 0)
    EdgesDrw2.name = Name+", sketched2 (EdgeType"+TypeDesc[EdgeType]+")"
    
    pdb.plug_in_edge (SketchedImg, EdgesDrw2, EdgeAmount, EdgeWrap, EdgeType)
    pdb.gimp_drawable_equalize (EdgesDrw2, False)
    
    HighPassDrw = EdgesDrw2.copy()
    SketchedImg.add_layer (HighPassDrw, 1)
    pdb.gimp_drawable_curves_spline (HighPassDrw, HISTOGRAM_VALUE, 10,
                                ( 0.00,0.00, 0.74,0.00, 0.75,0.01, 0.82,0.82, 1.00,1.00) )
    pdb.gimp_drawable_invert (EdgesDrw2, True)
    pdb.gimp_layer_add_alpha (EdgesDrw2)
    
    #pdb.gimp_display_new (SketchedImg)
    #pdb.gimp_context_pop()
    #return
    
    imagemask = pdb.gimp_layer_create_mask (EdgesDrw2, ADD_WHITE_MASK)
    EdgesDrw2.add_mask (imagemask)
    pdb.gimp_edit_copy (HighPassDrw)
    copypaste = pdb.gimp_edit_paste (imagemask, False)
    pdb.gimp_floating_sel_anchor (copypaste)
    SketchedImg.remove_layer (HighPassDrw)
    
    pdb.gimp_drawable_curves_spline (EdgesDrw2, HISTOGRAM_VALUE, 10,
                                ( 0.00,0.00, 0.74,0.00, 0.75,0.01, 0.82,0.82, 1.00,1.00) )
    
    EdgesDrw2.mode = LAYER_MODE_LINEAR_BURN   
    EdgesDrw2.opacity = 95.0
    
    # -------------------------------------------------------------------------
    # 2nd Edges (2) end
    
    #pdb.gimp_display_new (SketchedImg)
    #pdb.gimp_context_pop()
    #return
    
    # merge to have a single Edges layer    <<<<<<
    EdgesDrw = pdb.gimp_image_merge_down (SketchedImg, EdgesDrw2, CLIP_TO_IMAGE)
    EdgesDrw.mode = LAYER_MODE_LINEAR_BURN   
    EdgesDrw.opacity = 50.0
    
    #pdb.gimp_display_new (SketchedImg)
    #pdb.gimp_context_pop()
    #return
    
    DirtyOriginal = Original.copy()
    SketchedImg.add_layer (DirtyOriginal, 1)
    
    #[G'MIC] Dirty: fx_dirty 5,0,0,0,0,50,50
    pdb.plug_in_gmic_qt(SketchedImg, DirtyOriginal, 1,0 ,            
        "-v - -fx_dirty 3,0,0,0,0,50,50 ")
    
    #[G'MIC] Sketch: fx_sketchbw 3,45,180,120,4,0.026,0.5,0.75,0.1,0.7,3,6,0,1,4,0,50,50    
    pdb.plug_in_gmic_qt(SketchedImg, DirtyOriginal, 1,0 ,              
        "-v - -fx_sketchbw 3,45,180,60,4,0.029,0.5,0.75,0.1,0.7,3,6,0,1,4,0,50,50 ")     # mahvin 
    
    DirtySketch = SketchedImg.layers[0]
    DirtySketch.name = "Dirty Sketch"
    DirtySketch.mode = LAYER_MODE_GRAIN_MERGE        #######LUMA_DARKEN_ONLY    
    DirtySketch.opacity = 75.0  # over paper 
    
    EdgesDrw.mode = LAYER_MODE_LUMA_DARKEN_ONLY #MULTIPLY     ##### moved up
    
    EdgedDirtySketch = pdb.gimp_layer_new_from_visible (SketchedImg, SketchedImg,
                                                        "EdgedDirtySketch")
    SketchedImg.add_layer (EdgedDirtySketch, 0)
    EdgedDirtySketch.mode = LAYER_MODE_NORMAL         
    EdgedDirtySketch.opacity = 100.0  
    
    #pdb.gimp_display_new (SketchedImg)
    #pdb.gimp_context_pop()
    #return

    BG = pdb.gimp_layer_new (SketchedImg, rWidth, rHeight, RGB_IMAGE,
                             "Background", 100.0, LAYER_MODE_NORMAL)
    SketchedImg.add_layer (BG, 1)
    # colour selection
    if BGpredefcol == 0: pdb.gimp_context_set_foreground (BGusercol); po=" (paper user)"
    if BGpredefcol == 1: pdb.gimp_context_set_foreground (BLOSSOM); po=" (paper blossom)"
    if BGpredefcol == 2: pdb.gimp_context_set_foreground (CORAL); po=" (paper coral)"
    if BGpredefcol == 3: pdb.gimp_context_set_foreground (PAPAYA); po=" (paper papaya)"
    if BGpredefcol == 4: pdb.gimp_context_set_foreground (POPPY); po=" (paper poppy)"
    if BGpredefcol == 5: pdb.gimp_context_set_foreground (CURRY); po=" (paper curry)"
    if BGpredefcol == 6: pdb.gimp_context_set_foreground (SUNSHINE); po=" (paper sunshine)"
    if BGpredefcol == 7: pdb.gimp_context_set_foreground (CHARTREUSE); po=" (paper chartreuse)"
    if BGpredefcol == 8: pdb.gimp_context_set_foreground (MINT); po=" (paper mint)"
    if BGpredefcol == 9: pdb.gimp_context_set_foreground (CLOVER); po=" (paper clover)"
    if BGpredefcol == 10: pdb.gimp_context_set_foreground (SAGE); po=" (paper sage)"
    if BGpredefcol == 11: pdb.gimp_context_set_foreground (BLUEBELL); po=" (paper bluebell)"
    if BGpredefcol == 12: pdb.gimp_context_set_foreground (POOL); po=" (paper pool)"
    if BGpredefcol == 13: pdb.gimp_context_set_foreground (PLUM); po=" (paper plum)"
    if BGpredefcol == 14: pdb.gimp_context_set_foreground (CEMENT); po=" (paper cement)"
    if BGpredefcol == 15: pdb.gimp_context_set_foreground (GRAVEL); po=" (paper gravel)"
    if BGpredefcol == 16: pdb.gimp_context_set_foreground (PAPER_BAG); po=" (paper paper bag)"
    if BGpredefcol == 17: pdb.gimp_context_set_foreground (LUXE_BLUSH); po=" (paper luxe blush)"
    if BGpredefcol == 18: pdb.gimp_context_set_foreground (ANTIQUE_GOLD); po=" (paper antique gold)"
    if BGpredefcol == 19: pdb.gimp_context_set_foreground (SHIMMER_GOLD); po=" (paper shimmer gold)"
    if BGpredefcol == 20: pdb.gimp_context_set_foreground (SHIMMER_SILVER); po=" (paper shimmer silver)"
    # colour fill
    pdb.gimp_drawable_fill (BG, FILL_FOREGROUND)
    
    EdgedDirtySketch.mode = LAYER_MODE_MULTIPLY #LINEAR_BURN  #GRAIN_MERGE  # over paper        
    EdgedDirtySketch.opacity = 85.0  
    
    if outLook>0: # not only borders
        originalOnTop = Original.copy()
        SketchedImg.add_layer (originalOnTop, 0)
        PosterOnTop = PosterizedDrw.copy()
        SketchedImg.add_layer (PosterOnTop, 0)
        SketchOnTop = EdgedDirtySketch.copy()
        SketchedImg.add_layer (SketchOnTop, 0)
        
        originalOnTop.mode = LAYER_MODE_DODGE  
        originalOnTop.opacity = 30.0
        PosterOnTop.mode = LAYER_MODE_LINEAR_BURN
        PosterOnTop.opacity = 25.0
        SketchOnTop.mode = LAYER_MODE_OVERLAY
        SketchOnTop.opacity = 50.0
        
    PreOutcomeCol = pdb.gimp_layer_new_from_visible (SketchedImg, SketchedImg,
                                                     "Preliminary Outcome in Colours")
    SketchedImg.add_layer (PreOutcomeCol, 0)
    PreOutcomeCol.mode = LAYER_MODE_NORMAL  
    PreOutcomeCol.opacity = 100.0
    PreOutcomeGray = PreOutcomeCol.copy()
    SketchedImg.add_layer (PreOutcomeGray, 0)
    PreOutcomeGray.name = "Preliminary Outcome in Gray"
    gegl_c2g(SketchedImg, PreOutcomeGray)
    PreOutcomeGray.mode = LAYER_MODE_SCREEN  
    
    if outLook==1:
        PreOutcomeGray.opacity = 1.0
    if outLook==2:
        PreOutcomeGray.opacity = 40.0
    if outLook==3:
        PreOutcomeGray.opacity = 80.0
        PreOutcomeGray.mode = LAYER_MODE_NORMAL
        
    PreOutcomeMix = pdb.gimp_layer_new_from_visible (SketchedImg, SketchedImg,
                                                     "Preliminary Outcome Mix") 
    SketchedImg.add_layer (PreOutcomeMix, 0)
    PreOutcomeMix.mode = LAYER_MODE_NORMAL         
    PreOutcomeMix.opacity = 80.0  
    
    #[G'MIC] Canvas Texture: texturize_canvas 100,3,0.6
    pdb.plug_in_gmic_qt(SketchedImg, PreOutcomeMix, 1,2 ,
        "-v - -texturize_canvas 20,3,0.0 ")
    Canvas = SketchedImg.layers[0]
    Canvas.name = "Canvas"
    Canvas.mode = LAYER_MODE_NORMAL
    Canvas.opacity = 100.0
    # check mask
    if maskedlayer==True:
        pdb.gimp_image_raise_item_to_top (SketchedImg, OriginalMasked)
        originalmask = pdb.gimp_layer_get_mask (OriginalMasked)
        pdb.plug_in_vinvert (SketchedImg, originalmask)
    
    Outcome = pdb.gimp_layer_new_from_visible (SketchedImg, SketchedImg, "Outcome")
    SketchedImg.add_layer (Outcome, 0)
    look_options=["Simple sketch look","Vivid colours look","Light colours look",
                  "Rich gray look"]
    Outcome.name = st+" "+Name+" drawed in style D using option "+str(look_options[outLook])
    Outcome.name = Outcome.name+po 
    if outLook>1: # gray looks
        Outcome.name = Outcome.name+" (GEGL desat C2G)"
    pdb.gimp_item_set_visible (Outcome, False)
    #
    iratio = outimgWH*1.0/(SketchedImg.width+SketchedImg.height)*1.0
    if outimgWH==0: # original size
        pdb.gimp_image_scale (SketchedImg, inImage.width, inImage.height)
                              # back to original size
    else:
        iratio = float(outimgWH*1.0/(SketchedImg.width+SketchedImg.height)*1.0)
        pdb.gimp_image_scale (SketchedImg, int(SketchedImg.width*iratio),
                              int(SketchedImg.height*iratio))
                              # out to user chosen size
    if doFlatten == True:
        pdb.gimp_item_set_visible (Outcome, True)
        pdb.gimp_item_set_visible (Original, False)
        saved_name = Outcome.name
        Outcome = pdb.gimp_image_merge_visible_layers (SketchedImg, CLIP_TO_IMAGE)
        Outcome.name = saved_name
     
    pdb.gimp_display_new (SketchedImg)
    pdb.gimp_context_pop()
    
    return(Outcome)

### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset (inspired by kward1979uk+Paul Sherman)"
menu="<Image>/Diego/Drawing/"
copyrightYear="2020"
desc="Draw Effect Style D (with GEGL)"

register(
    "DrawStyleD",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    "Draw Effect Style D (with GEGL) ...",
    "*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_ADJUSTMENT, "outwh", "Output image Width+Heigth"+
                      "\n (0=same as input)", 4000, (0, 10000, 100)),
      (PF_OPTION, "precol", "Predefined BG colour", 0, ["*user (choose below)",
                                                        "BLOSSOM		250 198 204",
                                                        "CORAL		    255 182 173",
                                                        "PAPAYA		    246 139 105",
                                                        "POPPY		    251 175  95",
                                                        "CURRY		    250 207  99",
                                                        "SUNSHINE	    255 239 108",
                                                        "CHARTREUSE	    206 215  92",
                                                        "MINT		    210 239 219",
                                                        "CLOVER		    161 206  94",
                                                        "SAGE		    219 223 195",
                                                        "BLUEBELL	    174 209 233",
                                                        "POOL		    162 208 207",
                                                        "PLUM		    204 193 219",
                                                        "CEMENT		    227 223 214",
                                                        "GRAVEL		    178 177 165",
                                                        "PAPER_BAG	    197 171 137",
                                                        "LUXE_BLUSH	    255 242 236",
                                                        "ANTIQUE_GOLD	179 142  80",
                                                        "SHIMMER_GOLD	255 226 147",
                                                        "SHIMMER_SILVER	200 201 208",
                                                        ]),
      (PF_COLOR, "freecol", "*user BG colour"+
                            "\n (use light colours)", (150, 150, 150, 255)),
      (PF_OPTION, "look", "Type of Look", 1, ["Simple Sketch",
                                              "Vivid Colour Drawing",
                                              "Light Colour Drawing",
                                              "Rich Gray Drawing",
                                              ]),
      (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ],
    [
      (PF_DRAWABLE, "outdrawable", "Output drawable", None),
    ],
    DrawStyleD,
    menu=menu
    )

main()
